import java.awt.Color;
import java.awt.Polygon;
import java.awt.Graphics;
import javax.swing.JFrame;

public class Shape {
    
    
    // center
    
    int x, y, radius;
    String type;
    Color color;
    
    boolean isCircle;
    Polygon polygon = null;
    
    
    // outline
    
    public static Color OUTLINE = Color.BLACK;
    
    // constructor
    
    public Shape(double x, double y, double radius, String type, Color color) {
        this.x = (int) Math.round(x);
        this.y = (int) Math.round(y);
        this.radius = (int) Math.round(radius);
        this.type = type;
        this.color = color;
        
        isCircle = type.equals("Circle");
        if (isCircle) return;
        
        if (type.equals("Triangle")) {
            this.polygon = drawPolygon(3, 0, false);
            double h = radius * Math.sin(Math.toRadians(15)) / Math.sin(Math.toRadians(120));
            for (int i = 0; i < this.polygon.npoints; ++i) {
                this.polygon.ypoints[i] += h - 1; // "center"
            }
        }
        else if (type.equals("Square")) {
            this.polygon = drawPolygon(4, 45, false);
        }
        else if (type.equals("Diamond")) {
            this.polygon = drawPolygon(4, 0, false);
        }
        else if (type.equals("Pentagon")) {
            this.polygon = drawPolygon(5, 0, false);
        }
        else if (type.equals("Star")) {
            this.polygon = drawPolygon(10, 0, true);
        }
        else if (type.equals("Hexagon")) {
            this.polygon = drawPolygon(6, 0, false);
        }
        else if (type.equals("Heptagon")) {
            this.polygon = drawPolygon(7, 0, false);
        }
        else if (type.equals("Octagon")) {
            this.polygon = drawPolygon(8, 0, false);
        }
        // default
        else {
            System.out.println("ERROR " + type);
            System.exit(0);
            this.polygon = new Polygon();
        }

    }
    
    // check if shape contains point
    
    double cross(double a, double b, double c, double d, double e, double f) {
        return cross(a, b, c, d) + cross(c, d, e, f) + cross(e, f, a, b);
    }
    double cross(double a, double b, double c, double d) {
        return a * d - b * d;
    }
    
    public boolean containsPoint(double x, double y) {
        if (isCircle) {
            double dx = x - this.x;
            double dy = y - this.y;
            return dx * dx + dy * dy <= getCircleRadius() * getCircleRadius() + 1e-7;
        }
        else {
            
            // polygon
            boolean in = false;
            for (int i = 0, j = this.polygon.npoints - 1; i < this.polygon.npoints; j = i++) {
                double x1 = polygon.xpoints[j];
                double y1 = polygon.ypoints[j];
                double x2 = polygon.xpoints[i];
                double y2 = polygon.ypoints[i];
                if (Math.abs(cross(x1, y1, x, y, x2, y2)) <= 1) return true; // on the polygon
                in ^= (
                    ((y2 > y) != (y1 > y))
                    &&
                    x < (x2 - x1) * (y - y2) / (y2 - y1) + x2);
            }
            return in;
        }
    }
    
    // draw method
    
    public double circleScale = 0.85;
    
    public int getCircleRadius() {
        return (int) (radius * circleScale);
    }
    
    public void draw(Graphics g) {
        
        // fill
        g.setColor(color);
        
        if (isCircle)
            g.fillOval(x - getCircleRadius(), y - getCircleRadius(), getCircleRadius() * 2, getCircleRadius() * 2);
        else if (this.polygon != null)
            g.fillPolygon(this.polygon);
        
        
        // outline
        g.setColor(OUTLINE);
        
        if (isCircle)
            g.drawOval(x - getCircleRadius(), y - getCircleRadius(), getCircleRadius() * 2, getCircleRadius() * 2);
        else if (this.polygon != null)
            g.drawPolygon(this.polygon);
    }
    
    public int getInscribedRadius(int sides) {
        return (int) Math.round((double) radius / Math.cos(Math.toRadians(360.0 / sides / 2)));
    }
    
    public Polygon drawPolygon(int sides, double degree, boolean star) {
        degree -= 90;
        // System.out.printf("Drawing polygon with sides: %d %f %s\n", sides, degree, (star ? "STAR" : ""));
        
        
        // first point is (x, y) + (r * cos(d), r * sin(d))
        int xp[] = new int[sides];
        int yp[] = new int[sides];
        
        int rad = radius;
        
        for (int i = 0; i < sides; ++i) {
            xp[i] = (int) Math.round(this.x + ((star && i % 2 != 0) ? rad / 2 : rad) * Math.cos(Math.toRadians(degree + (i * 360.0 / sides))));
            yp[i] = (int) Math.round(this.y + ((star && i % 2 != 0) ? rad / 2 : rad) * Math.sin(Math.toRadians(degree + (i * 360.0 / sides))));
        }
        return new Polygon(xp, yp, sides);
    }
    
}