import java.util.*;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JComponent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;

/// ShapeCanvas is a canvas of shapes (duh)
/// extends JComponent instead of Canvas for
/// automatic back-buffering

public class ShapeCanvas extends JComponent {
    
    public ShapeCanvas(int rows, int columns, int shapeCount, int colorCount) {
        this.rows = rows;
        this.columns = columns;
        this.shapeCount = shapeCount;
        this.colorCount = colorCount;
    }
    
    // lists containing the determined rules and which are unlocked by the player
    public boolean[] rulesRevealed;
    public String[] ruleLog;
    
    // lists of all possible shapes and colors
    public static String[] SHAPES = new String[] {
        "Circle", "Triangle", "Square", "Diamond", "Pentagon", "Star", "Hexagon", "Heptagon", "Octagon"
    };
    
    public static Color[] COLORS = new Color[] {
        Color.CYAN, Color.ORANGE, Color.RED, Color.BLUE, Color.YELLOW, Color.GREEN, Color.MAGENTA, Color.PINK, new Color(0xC0C0C0), new Color(0x6C3082)
    
        //Cyan, Orange, Red, Blue, Yellow, Green, Magenta, Pink, Silver, Purple
    };
    
    public static HashMap<Color, String> EnglishColors = new HashMap<>();
    static {
        String[] names = new String[]{
            "Cyan", "Orange", "Red", "Blue", "Yellow", "Green", "Magenta", "Pink", "Silver", "Purple"
        };
        for (int i = 0; i < COLORS.length; ++i) {
            EnglishColors.put(COLORS[i], names[i]);
        }
    }
    
    // shape / color usage
    public int shapeCount = SHAPES.length;
    public int colorCount = COLORS.length;
    
    // members
    
    public int rows, columns;
    public Shape[][] shapes, original;
    
    // radius functions
    
    public int getRadius() {
        return Math.min(getWidth() / rows, getHeight() / columns) * 1 / 3;
        // return Math.min(getDistanceX(), getDistanceY()) * 1 / 2;
    }

    void init() {
        shapes = new Shape[rows][columns];
        original = new Shape[rows][columns];
        
        for (int i = 0; i < rows; ++i) {
            for (int j = 0; j < columns; ++j) {
                shapes[i][j] = generateShape(i, j);
            }
        }
        
        CopyBoard(shapes, original);
        
        MainGame.CurrentGame.checker.makePath();
        
        // add mouse clickers to the shapes
        this.addMouseListener(new MouseListener() {
            @Override
            public void mouseClicked(MouseEvent e) {
            }
            @Override
            public void mousePressed(MouseEvent e) {
            }
            @Override
            public void mouseReleased(MouseEvent e) {
                int x = e.getX();
                int y = e.getY();
                // get approximate shape
                
                int ai = x / (getWidth() / rows);
                int aj = y / (getHeight() / columns);
                if (shapes[ai][aj] != null && shapes[ai][aj].containsPoint(x, y))
                    PerformClick(ai, aj);
                
                /*
                 // brute force
                 for (int i = 0; i < rows; ++i) {
                 for (int j = 0; j < columns; ++j) {
                 if (shapes[i][j].containsPoint(x, y)) {
                 clickPoint(x, y);
                 return;
                 }
                 }
                 }
                 */
            }
            @Override
            public void mouseEntered(MouseEvent e) {}
            @Override
            public void mouseExited(MouseEvent e) {}
        });
    }
    
    void PerformClick(int i, int j) {
        Shape currentShape = shapes[i][j];
        
        // do stuff here
        Checker checker = MainGame.getChecker();
        Rule rule = checker.peek();
        
        if (rule.meets(currentShape)) {
            // wtf yes
            checker.pop();
            shapes[i][j] = null;
            MainGame.update();
        }
        else {
            JOptionPane.showMessageDialog(MainGame.CurrentGame, "Invalid move.");
            checker.reset();
        }
    }
    
    // painter
    
    @Override
    public void paint(Graphics g) {
        // background
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, getWidth(), getHeight());
        
        if (shapes == null) {
            init();
        }
        for (int i = 0; i < rows; ++i) {
            for (int j = 0; j < columns; ++j) {
                if (shapes[i][j] != null)
                    shapes[i][j].draw(g);
            }
        }
    }
    
    public Shape generateShape(int i, int j)
    {
        int shape = (int)(Math.random() * Math.min(SHAPES.length, shapeCount));
        int color = (int)(Math.random() * Math.min(COLORS.length, colorCount));
        
        Shape spawned = new Shape(
            i * getWidth() / rows + getWidth() / (rows * 2),
            j * getHeight() / rows + getHeight() / (rows * 2),
            getRadius(),
            SHAPES[shape],
            COLORS[color]
        );
        
        // Shape spawned = new Shape(getPadding() + getRadius() + i * (getWidth() - 2 * getPadding()) / (rows), getPadding() + getRadius() + j * (getHeight() - 2 * getPadding()) / (columns), getRadius(), SHAPES[shape], COLORS[color]);
        
        return spawned;
    }
    
    static void CopyBoard(Shape[][] a, Shape[][] b) {
        int n = a.length;
        int m = a[0].length;
        for (int i = 0; i < n; ++i) {
            for (int j = 0; j < m; ++j) {
                b[i][j] = a[i][j];
            }
        }
    }
    
    public void reset() {
        CopyBoard(original, shapes);
        repaint();
    }
    
}